Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
var ErrNotInProvider = errors.New("a local listener must be created with the ListenerProvider.New method")
var ErrNotNotificationType = errors.New("a notification must be of type listener.Notification")
Functions ¶
This section is empty.
Types ¶
type Listener ¶
type Listener struct {
// contains filtered or unexported fields
}
type ListenerFactory ¶
type ListenerFactory struct {
// contains filtered or unexported fields
}
func NewListenerFactory ¶
func NewListenerFactory(llf *ListenerProvider) *ListenerFactory
func (*ListenerFactory) New ¶
func (l *ListenerFactory) New(channelName string, matcher listener.TypeMatcher) listener.Listener
func (*ListenerFactory) Shutdown ¶
func (l *ListenerFactory) Shutdown()
type ListenerProvider ¶
type ListenerProvider struct {
// contains filtered or unexported fields
}
func NewListenerProvider ¶
func NewListenerProvider(args ListenerProviderArgs) *ListenerProvider
func (*ListenerProvider) New ¶
func (l *ListenerProvider) New(name string) *Listener
Only intended to be called from listenerfactory.go's `New` method.
func (*ListenerProvider) Notify ¶
func (l *ListenerProvider) Notify(channel string, n interface{})
Only intended to be called from the store conn_base.go's `Notify` method. Please use the store method and don't call this directly.
type ListenerProviderArgs ¶
type ListenerProviderArgs struct {
}
Click to show internal directories.
Click to hide internal directories.